/**
 * Copyright Marcus Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of LibDesQ (https://gitlab.com/DesQ/libdesq)
 * This library contains various  Core and Gui classes which are used
 * through out the DesQ Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#pragma once

#include "desq/Globals.hpp"
#include <QtGui>

typedef QMap<QString, QVariant> SettingsMap;

class QDBusInterface;

namespace DesQ {
    class Settings;
}

class DesQ::Settings : public QObject {
    Q_OBJECT;

    public:

        /*
         * @project -> Project like DesQ or Oyster
         * @app -> Settings of a specific app: Like Files, Term, Shell, etc...
         */
        Settings( QString app );

        struct DesQProxy {
            QVariant value;

            operator int() const {
                return value.toInt();
            }

            operator float() const {
                return value.toReal();
            }

            operator double() const {
                return value.toDouble();
            }

            operator bool() const {
                return value.toBool();
            }

            operator QString() const {
                return value.toString();
            }

            operator QStringList() const {
                return value.value<QStringList>();
            }

            operator QSize() const {
                return value.toSize();
            }

            operator QFont() const {
                return value.value<QFont>();
            }

            operator QRect() const {
                return value.toRect();
            }

            operator QVariant() const {
                return value;
            }
        };

        DesQProxy value( const QString& key );
        void setValue( const QString& key, QVariant value );

    private:
        QSettings *user, *system;
        QFileSystemWatcher *watcher;

        SettingsMap userMap;

        QVariant rawValue( const QString& key );

    private Q_SLOTS:
        void emitSettings( QSettings * );

    Q_SIGNALS:
        void settingChanged( QString, QVariant );
};
